@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2020 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -211,14 +211,18 @@ class LabelBatchHandler(BatchHandler):
         def append(entry, i):
             upc = entry['upc'].strip()
             if upc:
-                upc = GPC(upc, calc_check_digit=self.calc_check_digit)
-                product = api.get_product_by_upc(session, upc)
-                if product:
-                    if batch.static_prices and entry['regular_price']:
-                        product._batch_price = decimal.Decimal(entry['regular_price'])
-                    products.append(product)
+                try:
+                    upc = GPC(upc, calc_check_digit=self.calc_check_digit)
+                except ValueError:
+                    pass
                 else:
-                    log.warning("product not found: {}".format(upc))
+                    product = api.get_product_by_upc(session, upc)
+                    if product:
+                        if batch.static_prices and entry['regular_price']:
+                            product._batch_price = decimal.Decimal(entry['regular_price'])
+                        products.append(product)
+                    else:
+                        log.warning("product not found: {}".format(upc))
 
         self.progress_loop(append, data, progress,
                            message="Reading data from CSV file")
