@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2017 Lance Edgar
+#  Copyright © 2010-2019 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,9 +24,10 @@
 Handler for Vendor Invoice batches
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
 import os
+import decimal
 
 from sqlalchemy import orm
 
@@ -153,12 +154,23 @@ class VendorInvoiceHandler(BatchHandler):
         if not row.case_cost and row.unit_cost and row.case_quantity:
             row.case_cost = row.unit_cost * row.case_quantity
 
-        if not product.cost_for_vendor(self.vendor):
+        cost = product.cost_for_vendor(self.vendor)
+        if not cost:
             row.status_code = row.STATUS_COST_NOT_IN_DB
-        elif not row.case_quantity:
-            row.status_code = row.STATUS_NO_CASE_QUANTITY
         else:
-            row.status_code = row.STATUS_OK
+
+            # calculate unit cost diff, if we have two costs to compare
+            if cost.unit_cost is not None and row.unit_cost is not None:
+                # note, we must round early here to avoid "false diff"
+                row.unit_cost_diff = (row.unit_cost - cost.unit_cost).quantize(
+                    decimal.Decimal('0.00001'))
+
+            if not row.case_quantity:
+                row.status_code = row.STATUS_NO_CASE_QUANTITY
+            elif row.unit_cost_diff:
+                row.status_code = row.STATUS_UNIT_COST_DIFFERS
+            else:
+                row.status_code = row.STATUS_OK
 
     def cognize_purchase_order(self, session, invoice, purchase, progress=None):
         """
