@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2020 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -667,6 +667,13 @@ def make_config(files=None, usedb=None, preferdb=None, env=None, winsvc=None, ex
     config = RattailConfig(files, usedb=usedb, preferdb=preferdb)
     config.configure_logging()
 
+    # turn on display of rattail deprecation warnings by default
+    # TODO: this should be configurable, and possibly live elsewhere?
+    warnings.filterwarnings('default', category=DeprecationWarning,
+                            module=r'^rattail')
+    warnings.filterwarnings('default', category=DeprecationWarning,
+                            module=r'^tailbone')
+
     if config.getbool('rattail', 'suppress_psycopg2_wheel_warning', usedb=False):
         # TODO: revisit this, does it require action from us?
         # suppress this warning about psycopg2 wheel; not sure what it means yet
