@@ -52,33 +52,75 @@ class ToRattail(ToSQLAlchemy):
     extension_attr = None
     extension_fields = []
 
+    extensions = {}
+
     @property
     def simple_fields(self):
-        return super(ToRattail, self).simple_fields + self.extension_fields
+        fields = list(super(ToRattail, self).simple_fields)
+
+        # extension fields, original style
+        fields.extend(self.extension_fields)
+
+        # extension fields, newer flexible style
+        for name, ext_fields in six.iteritems(self.extensions):
+            fields.extend(ext_fields)
+
+        return fields
 
     def cache_query_options(self):
         options = super(ToRattail, self).cache_query_options() or []
-        # eager join extension table if applicable
+
+        # extension fields, original style
         if self.extension_attr and self.fields_active(self.extension_fields):
-            options.append(orm.joinedload(getattr(self.model_class, self.extension_attr)))
+            options.append(orm.joinedload(
+                getattr(self.model_class, self.extension_attr)))
+
+        # extension fields, newer flexible style
+        for name, ext_fields in six.iteritems(self.extensions):
+            options.append(orm.joinedload(
+                getattr(self.model_class, name)))
+
         return options
 
     def normalize_local_object(self, obj):
         data = super(ToRattail, self).normalize_local_object(obj)
-        if data and self.extension_fields:
+        if data:
+
+            # extension fields, original style
             for field in self.extension_fields:
                 if field in self.fields:
                     data[field] = getattr(obj, field)
-        return data
+
+            # extension fields, newer flexible style
+            for name, ext_fields in six.iteritems(self.extensions):
+                for field in ext_fields:
+                    if field in self.fields:
+                        data[field] = getattr(obj, field)
+
+            return data
 
     def update_object(self, obj, host_data, local_data=None):
         obj = super(ToRattail, self).update_object(obj, host_data, local_data)
-        if obj and self.extension_fields:
+        if obj:
+
+            # extension fields, original style
             for field in self.extension_fields:
                 if field in self.fields:
-                    if not local_data or field not in local_data or local_data[field] != host_data[field]:
+                    if (not local_data
+                        or field not in local_data
+                        or local_data[field] != host_data[field]):
                         setattr(obj, field, host_data[field])
-        return obj
+
+            # extension fields, newer flexible style
+            for name, ext_fields in six.iteritems(self.extensions):
+                for field in ext_fields:
+                    if field in self.fields:
+                        if (not local_data
+                            or field not in local_data
+                            or local_data[field] != host_data[field]):
+                            setattr(obj, field, host_data[field])
+
+            return obj
 
 
 class GlobalMixin(object):
@@ -2413,7 +2455,7 @@ class ProductCostImporter(ToRattail):
                             product.costs.append(cost)
                             product.costs.reorder()
                         else:
-                            log.warning("cannot un-prefer cost, as product has only the one: {}".format(cost))
+                            log.warning("cannot un-prefer cost for product %s as it has only the one: {}".format(product.uuid, cost))
                 else:
                     if not product.costs:
                         log.warning("new cost will be preferred, as product has only the one: {}".format(self.get_key(data)))
