@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2020 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -39,7 +39,10 @@ class FromRattailHandler(FromSQLAlchemyHandler):
     """
     Base class for import handlers which target a Rattail database on the local side.
     """
-    host_title = "Rattail"
+
+    @property
+    def host_title(self):
+        return self.config.app_title(default="Rattail")
 
     def make_host_session(self):
         return Session()
@@ -49,7 +52,10 @@ class ToRattailHandler(ToSQLAlchemyHandler):
     """
     Base class for import handlers which target a Rattail database on the local side.
     """
-    local_title = "Rattail"
+
+    @property
+    def local_title(self):
+        return self.config.app_title(default="Rattail")
 
     def make_session(self):
         kwargs = {}
