@@ -18,7 +18,8 @@ class TestRecordChangesFunc(TestCase):
 
     def test_session_class(self):
         Session = orm.sessionmaker()
-        self.assertRaises(KeyError, Session.kw.__getitem__, 'rattail_record_changes')
+        if hasattr(Session, 'kw'):
+            self.assertRaises(KeyError, Session.kw.__getitem__, 'rattail_record_changes')
         self.assertRaises(AttributeError, getattr, Session, 'rattail_record_changes')
         changes.record_changes(Session)
         self.assertTrue(Session.rattail_record_changes)
@@ -179,9 +180,6 @@ class TestChangeRecorder(DataTestCase):
         self.assertEqual(self.session.query(model.Change).count(), 0)
 
         # ignore batch object (TODO: redundant?)
-        self.assertFalse(recorder.record_rattail_change(self.session, model.Batch()))
-        self.assertFalse(recorder.record_rattail_change(self.session, model.BatchColumn()))
-        self.assertFalse(recorder.record_rattail_change(self.session, model.BatchRow()))
         self.assertFalse(recorder.record_rattail_change(self.session, model.BatchMixin()))
         self.assertFalse(recorder.record_rattail_change(self.session, model.BatchRowMixin()))
         self.assertEqual(self.session.query(model.Change).count(), 0)
@@ -239,11 +237,6 @@ class TestChangeRecorderLegacy(TestCase):
     #     # don't record changes for changes
     #     self.assertFalse(recorder.record_change(session, model.Change()))
 
-    #     # don't record changes for batch data
-    #     self.assertFalse(recorder.record_change(session, model.Batch()))
-    #     self.assertFalse(recorder.record_change(session, model.BatchColumn()))
-    #     self.assertFalse(recorder.record_change(session, model.BatchRow()))
-
     #     # don't record changes for objects with no uuid attribute
     #     self.assertFalse(recorder.record_change(session, object()))
 
